#!/usr/bin/env -S PYTHONPATH=../../../tools/extract-utils python3
#
# SPDX-FileCopyrightText: 2024 The LineageOS Project
# SPDX-License-Identifier: Apache-2.0
#

from extract_utils.file import File
from extract_utils.fixups_blob import (
    BlobFixupCtx,
    blob_fixup,
    blob_fixups_user_type,
)
from extract_utils.fixups_lib import (
    lib_fixup_remove,
    lib_fixups,
    lib_fixups_user_type,
)
from extract_utils.main import (
    ExtractUtils,
    ExtractUtilsModule,
)
from extract_utils.tools import (
    llvm_objdump_path,
)
from extract_utils.utils import (
    run_cmd,
)

namespace_imports = [
    'device/xiaomi/marble',
    'hardware/qcom-caf/sm8450',
    'hardware/xiaomi',
    "vendor/qcom/opensource/dataservices",
    "vendor/qcom/opensource/commonsys-intf/display",
    "vendor/qcom/opensource/commonsys/display",
    "hardware/qcom-caf/wlan"
]


def lib_fixup_vendor_suffix(lib: str, partition: str, *args, **kwargs):
    return f'{lib}_{partition}' if partition == 'vendor' else None


lib_fixups: lib_fixups_user_type = {
    **lib_fixups,
    (
        'com.qualcomm.qti.dpm.api@1.0',
        'vendor.qti.diaghal@1.0',
        'vendor.qti.hardware.dpmservice@1.0',
        'vendor.qti.hardware.qccsyshal@1.0',
        'vendor.qti.hardware.qccsyshal@1.1',
        'vendor.qti.hardware.qccvndhal@1.0',
        'vendor.qti.hardware.wifidisplaysession@1.0',
        'vendor.qti.imsrtpservice@3.0',
    ): lib_fixup_vendor_suffix,
    (
        'libpalclient',
        'libwpa_client',
        'libagm',
        'libar-acdb',
        'libar-gsl'
        'libar-gpr',
        'libar-pal',
        'libats',
        'liblx-ar_util'
        'liblx-osal',
        'libpalclient',
        'libmcs',
        'lib_bt_bundle',
        'lib_bt_aptx',
        'lib_bt_ble',
        'lib_bt_lhdc',
        'vendor.qti.hardware.pal@1.0-impl',
        'audio.primary.taro',
    ): lib_fixup_remove,
}


def blob_fixup_return_1(
    ctx: BlobFixupCtx,
    file: File,
    file_path: str,
    symbol: str,
    *args,
    **kwargs,
):
    for line in run_cmd(
        [
            llvm_objdump_path,
            '--dynamic-syms',
            file_path,
        ]
    ).splitlines():
        if line.endswith(f' {symbol}'):
            offset, _ = line.split(maxsplit=1)

            with open(file_path, 'rb+') as f:
                f.seek(int(offset, 16))
                f.write(b'\x01\x00\xa0\xe3')  # mov r0, #1
                f.write(b'\x1e\xff\x2f\xe1')  # bx lr

            break


blob_fixups: blob_fixups_user_type = {
    (
        'vendor/etc/camera/marble_enhance_motiontuning.xml',
        'vendor/etc/camera/marble_motiontuning.xml',
    ): blob_fixup().regex_replace('xml=version', 'xml version'),
    'vendor/etc/camera/pureView_parameter.xml': blob_fixup().regex_replace(
        r'=([0-9]+)>', r'="\1">'
    ),
    ('system_ext/lib/libwfdnative.so', 'system_ext/lib64/libwfdnative.so'): blob_fixup()
        .add_needed('libbinder_shim.so')
        .add_needed('libinput_shim.so')
        .replace_needed('android.hidl.base@1.0.so', 'libhidlbase.so'),
    ('system_ext/lib/libwfdservice.so', 'system_ext/lib64/libwfdservice.so'): blob_fixup()
        .replace_needed('android.media.audio.common.types-V2-cpp.so', 'android.media.audio.common.types-V4-cpp.so'),
    ('system_ext/lib64/libwfdmmsrc_system.so', 'system_ext/lib64/libwfdmmsrc_system.so'): blob_fixup()
        .add_needed('libgui_shim.so'),
    ('vendor/bin/qcc-trd', 'vendor/lib/libqtr_sdk.so', 'vendor/lib64/libqtr_sdk.so'): blob_fixup()
        .replace_needed('libgrpc++_unsecure.so', 'libgrpc++_unsecure_prebuilt.so'),
    ('vendor/bin/hw/android.hardware.gnss-aidl-service-qti', 'vendor/lib/hw/android.hardware.gnss-aidl-impl-qti.so', 'vendor/lib/libgarden.so', 'vendor/lib/libgarden_haltests_e2e.so', 'vendor/lib64/hw/android.hardware.gnss-aidl-impl-qti.so', 'vendor/lib64/libgarden.so', 'vendor/lib64/libgarden_haltests_e2e.so'): blob_fixup()
        .replace_needed('android.hardware.gnss-V1-ndk_platform.so', 'android.hardware.gnss-V1-ndk.so'),
    ('vendor/bin/hw/android.hardware.security.keymint-service-qti', 'vendor/lib/libqtikeymint.so', 'vendor/lib64/libqtikeymint.so'): blob_fixup()
        .replace_needed('android.hardware.security.keymint-V1-ndk_platform.so', 'android.hardware.security.keymint-V1-ndk.so')
        .replace_needed('android.hardware.security.secureclock-V1-ndk_platform.so', 'android.hardware.security.secureclock-V1-ndk.so')
        .replace_needed('android.hardware.security.sharedsecret-V1-ndk_platform.so', 'android.hardware.security.sharedsecret-V1-ndk.so')
        .add_needed('android.hardware.security.rkp-V1-ndk.so'),
    ('vendor/etc/media_codecs_cape.xml', 'vendor/etc/media_codecs_cape_vendor.xml'): blob_fixup()
        .regex_replace('.*media_codecs_(google_audio|google_c2|google_telephony|google_video|vendor_audio).*\n', ''),
    ('vendor/lib/libarcsoft_dark_vision_raw.so', 'vendor/lib/libarcsoft_high_dynamic_range_v5.so', 'vendor/lib64/libarcsoft_dark_vision_raw.so', 'vendor/lib64/libarcsoft_high_dynamic_range_v5.so'): blob_fixup()
        .clear_symbol_version('remote_handle_close')
        .clear_symbol_version('remote_handle_invoke')
        .clear_symbol_version('remote_handle_open')
        .clear_symbol_version('remote_register_buf_attr')
        .clear_symbol_version('remote_register_buf'),
    ('vendor/lib/libcamximageformatutils.so', 'vendor/lib64/libcamximageformatutils.so'): blob_fixup()
        .replace_needed('vendor.qti.hardware.display.config-V2-ndk_platform.so', 'vendor.qti.hardware.display.config-V2-ndk.so'),
    ('vendor/lib/vendor.libdpmframework.so', 'vendor/lib64/vendor.libdpmframework.so'): blob_fixup()
        .add_needed('libhidlbase_shim.so'),
    ('vendor/lib/libmorpho_video_stabilizer.so', 'vendor/lib64/libmorpho_video_stabilizer.so'): blob_fixup()
        .add_needed('libutils.so'),
    ('vendor/lib/libqcodec2_core.so', 'vendor/lib64/libqcodec2_core.so'): blob_fixup()
        .add_needed('libcodec2_shim.so'),
    ('vendor/lib/vendor.qti.hardware.qxr-V1-ndk_platform.so', 'vendor/lib64/vendor.qti.hardware.qxr-V1-ndk_platform.so'): blob_fixup()
        .replace_needed('android.hardware.common-V2-ndk_platform.so', 'android.hardware.common-V2-ndk.so'),
    ('vendor/lib/libqcrilNr.so', 'vendor/lib/libril-db.so', 'vendor/lib64/libqcrilNr.so', 'vendor/lib64/libril-db.so'): blob_fixup()
        .binary_regex_replace(rb'persist\.vendor\.radio\.poweron_opt', b'persist.vendor.radio.poweron_ign'),
    (
        'vendor/lib/hw/com.qti.chi.override.so',
        'vendor/lib64/hw/com.qti.chi.override.so',
        'vendor/lib/libcamxcommonutils.so',
        'vendor/lib64/libcamxcommonutils.so',
        'vendor/lib/libmialgoengine.so',
        'vendor/lib64/libmialgoengine.so',
    ): blob_fixup().add_needed('libprocessgroup_shim.so'),
    (
        'vendor/lib/libTrueSight.so',
        'vendor/lib/libalAILDC.so',
        'vendor/lib/libalLDC.so',
        'vendor/lib/libalhLDC.so',
        'vendor/lib64/libTrueSight.so',
        'vendor/lib64/libalAILDC.so',
        'vendor/lib64/libalLDC.so',
        'vendor/lib64/libalhLDC.so',
    ): blob_fixup()
        .clear_symbol_version('AHardwareBuffer_allocate')
        .clear_symbol_version('AHardwareBuffer_describe')
        .clear_symbol_version('AHardwareBuffer_lock')
        .clear_symbol_version('AHardwareBuffer_lockPlanes')
        .clear_symbol_version('AHardwareBuffer_release')
        .clear_symbol_version('AHardwareBuffer_unlock'),
    ('vendor/bin/hw/dolbycodec2'): blob_fixup()
        .add_needed('libstagefright_foundation-v33.so'),
    ('vendor/lib/c2.dolby.client.so', 'vendor/lib64/c2.dolby.client.so') : blob_fixup()
        .add_needed('libcodec2_hidl_shim.so'),
    ('vendor/bin/hw/android.hardware.identity-service-qti', 'vendor/lib/libqtiidentitycredential.so', 'vendor/lib64/libqtiidentitycredential.so'): blob_fixup()
        .replace_needed('android.hardware.identity-V3-ndk_platform.so', 'android.hardware.identity-V3-ndk.so')
        .replace_needed('android.hardware.keymaster-V3-ndk_platform.so', 'android.hardware.keymaster-V3-ndk.so'),
    (
        'vendor/lib/nfc_nci.nqx.default.hw.v1.so',
        'vendor/lib64/nfc_nci.nqx.default.hw.v1.so',
    ): blob_fixup()
        .add_needed('libbase_shim.so'),
    'vendor/bin/hw/vendor.xiaomi.hardware.vibratorfeature.service': blob_fixup()
        .replace_needed('android.hardware.vibrator-V1-ndk_platform.so', 'android.hardware.vibrator-V1-ndk.so'),
}  # fmt: skip

module = ExtractUtilsModule(
    'marble',
    'xiaomi',
    blob_fixups=blob_fixups,
    lib_fixups=lib_fixups,
    namespace_imports=namespace_imports,
)

module.add_proprietary_file('proprietary-files-phone.txt').add_copy_files_guard(
    'TARGET_IS_TABLET', 'true', invert=True
)

if __name__ == '__main__':
    utils = ExtractUtils.device(module)
    utils.run()
